DEFINT A-Z

' SOUNDEX.BAS: Convert text to Soundex code

' Written by Steve York (CIS:71055,1144)
' January 22, 1996

' This routine may used and distributed freely as long as you keep
' these comments about where it came from.

' No guarantees on anything. Use it at your own risk.

' Code logic based on Knuth's "The Art of Computer Programming", vol.3, p.392
' Special thanks to Myrna Larson and Phillip Cain for references.

' This code has been tested to be compatible with the following compilers:

' Microsoft Basic Professional Development System v7.1
' Microsoft Visual Basic for DOS v1.0
' Microsoft Visual Basic for Windows v3.0

' $INCLUDE: 'SOUNDEX.BI'

FUNCTION sSoundex$ (sWordSrc AS STRING)

    ' This routine will return a Soundex code for a string passed to it.
    ' The result will always be a 4 byte string. Empty strings will
    ' return as a SPACE character, followed by 3 zeros. (" 000")

    DIM sLetterWrk  AS STRING
    DIM sWordWrk    AS STRING
    DIM sCodeWrk    AS STRING
    DIM sCodeHld    AS STRING
    DIM sCodeDst    AS STRING * 4
    DIM iLetterIdx  AS INTEGER
    DIM iCodePtr    AS INTEGER

    sWordWrk = UCASE$(LTRIM$(RTRIM$(sWordSrc))) ' Assure case is all the same
    RSET sCodeDst = LEFT$(sWordWrk, 1) + "000"  ' Init with first letter and zeros
    iCodePtr = 1                                ' init pointer into dest

    FOR iLetterIdx = 1 TO LEN(sWordWrk)

        sLetterWrk = MID$(sWordWrk, iLetterIdx, 1)          ' Get character to convert
        SELECT CASE sLetterWrk                              ' Translate letters to codes
            CASE "A", "E", "H", "I", "O", "U", "W", "Y":    sCodeWrk = ""
            CASE "B", "F", "P", "V":                        sCodeWrk = "1"
            CASE "C", "G", "J", "K", "Q", "S", "X", "Z":    sCodeWrk = "2"
            CASE "D", "T":                                  sCodeWrk = "3"
            CASE "L":                                       sCodeWrk = "4"
            CASE "M", "N":                                  sCodeWrk = "5"
            CASE "R":                                       sCodeWrk = "6"
            CASE ELSE:                                      sCodeWrk = ""
        END SELECT

        IF iLetterIdx > 1 THEN                              ' Start w/2nd char, but retain 1st for dup chk
            IF sCodeWrk <> sCodeHld THEN                    ' Eliminate doubles
                IF sCodeWrk <> "" THEN                      ' Code assigned?
                    iCodePtr = iCodePtr + 1                 ' Find next position
                    MID$(sCodeDst, iCodePtr, 1) = sCodeWrk  ' Move in the code
                END IF
                IF iCodePtr = 4 THEN                        ' All positions filled?
                    EXIT FOR                                ' Don't need any more
                ELSE
                    sCodeHld = sCodeWrk                     ' Retain for double check
                END IF
            END IF
        ELSE
            sCodeHld = sCodeWrk     ' Just retain 1st code for double check
        END IF

    NEXT iLetterIdx

    sSoundex = sCodeDst

END FUNCTION

